@@ -2,14 +2,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at <http://mozilla.org/MPL/2.0/>. */
 
-// @flow
-
 import { getSymbols, getSource, getSelectedFrame, getCurrentThread } from ".";
 import { findClosestClass } from "../utils/ast";
 
-import type { State } from "../reducers/types";
-
-export function inComponent(state: State): ?string {
+export function inComponent(state) {
   const thread = getCurrentThread(state);
   const selectedFrame = getSelectedFrame(state, thread);
   if (!selectedFrame) {
