@@ -5,10 +5,8 @@
 
 /* global __dirname */
 
+const sharedJestConfig = require(`${__dirname}/../../../shared/test-helpers/shared-jest.config`);
+
 module.exports = {
-  verbose: true,
-  moduleNameMapper: {
-    // Map all require("devtools/...") to the real devtools root.
-    "^devtools\\/(.*)": `${__dirname}/../../../../$1`,
-  },
+  ...sharedJestConfig,
 };
