@@ -25,15 +25,25 @@ Services.prefs.setIntPref(
   4096
 );
 
+// Do not trunacate the blocked-uri in CSP reports for frame navigations.
+Services.prefs.setBoolPref(
+  "security.csp.truncate_blocked_uri_for_frame_navigations",
+  false
+);
+
 // ExtensionContent.jsm needs to know when it's running from xpcshell,
 // to use the right timeout for content scripts executed at document_idle.
 ExtensionTestUtils.mockAppInfo();
 
-const server = createHttpServer();
+const server = createHttpServer({
+  hosts: ["example.com", "csplog.example.net"],
+});
+
 server.registerDirectory("/data/", do_get_file("data"));
 
 var gContentSecurityPolicy = null;
 
+const BASE_URL = `http://example.com`;
 const CSP_REPORT_PATH = "/csp-report.sjs";
 
 /**
@@ -54,8 +64,6 @@ function registerStaticPage(path, content) {
   });
 }
 
-const BASE_URL = `http://localhost:${server.identity.primaryPort}`;
-
 /**
  * A set of tags which are automatically closed in HTML documents, and
  * do not require an explicit closing tag.
@@ -544,7 +552,7 @@ function testInlineCSS() {
 
       browser.test.assertThrows(
         () => style.sheet.wrappedJSObject.cssRules,
-        /operation is insecure/,
+        /Not allowed to access cross-origin stylesheet/,
         "Page content should not be able to access extension-generated CSS rules"
       );
 
@@ -682,10 +690,13 @@ function injectElements(tests, baseOpts) {
 
         for (let test of tests) {
           let { elem, srcElem, src } = createElement(test, opts);
+
           document.body.appendChild(elem);
           window.wrappedJSObject.elem = srcElem;
           window.wrappedJSObject.eval(
-            `elem.setAttribute(${uneval(test.srcAttr)}, ${uneval(src)})`
+            `elem.setAttribute(${JSON.stringify(
+              test.srcAttr
+            )}, ${JSON.stringify(src)})`
           );
         }
 
@@ -711,7 +722,9 @@ function injectElements(tests, baseOpts) {
           let { elem, srcElem, src } = createElement(test, opts);
           window.wrappedJSObject.elem = srcElem;
           window.wrappedJSObject.eval(
-            `elem.setAttribute(${uneval(test.srcAttr)}, ${uneval(src)})`
+            `elem.setAttribute(${JSON.stringify(
+              test.srcAttr
+            )}, ${JSON.stringify(src)})`
           );
           document.body.appendChild(elem);
         }
@@ -729,7 +742,9 @@ function injectElements(tests, baseOpts) {
           document.body.appendChild(elem);
           window.wrappedJSObject.elem = srcElem;
           window.wrappedJSObject.eval(
-            `elem.setAttribute(${uneval(test.srcAttr)}, ${uneval(src)})`
+            `elem.setAttribute(${JSON.stringify(
+              test.srcAttr
+            )}, ${JSON.stringify(src)})`
           );
         }
       }
@@ -821,6 +836,8 @@ function computeBaseURLs(tests, expectedSources, forbiddenSources = {}) {
 
   function* iterSources(test, sources) {
     for (let [source, attrs] of Object.entries(sources)) {
+      // if a source defines attributes (e.g. liveSrc in PAGE_SOURCES etc.) then all
+      // attributes in the source must be matched by the test (see const TEST).
       if (Object.keys(attrs).every(attr => attrs[attr] === test[attr])) {
         yield `${BASE_URL}/${test.src}?source=${source}`;
       }
@@ -865,9 +882,14 @@ function computeBaseURLs(tests, expectedSources, forbiddenSources = {}) {
  *        If true, a strict CSP is enabled for this page, and inline page
  *        sources should be blocked. URLs present in these sources will not be
  *        expected to generate a CSP report, the inline sources themselves will.
+ * @param {boolean} [contentCspEnabled = false]
  * @returns {RequestedURLs}
  */
-function computeExpectedForbiddenURLs({ urls, sources }, cspEnabled = false) {
+function computeExpectedForbiddenURLs(
+  { urls, sources },
+  cspEnabled = false,
+  contentCspEnabled = false
+) {
   let expectedURLs = new Set();
   let forbiddenURLs = new Set();
   let blockedURLs = new Set();
@@ -881,6 +903,10 @@ function computeExpectedForbiddenURLs({ urls, sources }, cspEnabled = false) {
       } else {
         blockedURLs.add(baseURL);
       }
+    } else if (contentCspEnabled && origin === "contentScript") {
+      if (inline) {
+        forbiddenURLs.add(baseURL);
+      }
     } else {
       expectedURLs.add(baseURL);
     }
@@ -1003,7 +1029,7 @@ function awaitCSP(urlsPromise) {
         if (blockedURLs.has(baseURL)) {
           blockedURLs.delete(baseURL);
 
-          info(`Got CSP report for forbidden URL ${origURL}`);
+          ok(true, `Got CSP report for forbidden URL ${origURL}`);
         }
       }
 
@@ -1012,7 +1038,8 @@ function awaitCSP(urlsPromise) {
         if (blockedSources.has(source)) {
           blockedSources.delete(source);
 
-          info(
+          ok(
+            true,
             `Got CSP report for forbidden inline source ${JSON.stringify(
               source
             )}`
@@ -1021,7 +1048,7 @@ function awaitCSP(urlsPromise) {
       }
 
       if (!blockedURLs.size && !blockedSources.size) {
-        info("Got all expected CSP reports");
+        ok(true, "Got all expected CSP reports");
         resolve();
       }
     }
@@ -1063,6 +1090,9 @@ const TESTS = [
   },
   // TODO: <frame> element, which requires a frameset document.
   {
+    // the blocked-uri for frame-navigations is the pre-path URI. For the
+    // purpose of this test we do not strip the blocked-uri by setting the
+    // preference 'truncate_blocked_uri_for_frame_navigations'
     element: ["iframe", {}],
     src: "iframe.html",
   },
@@ -1139,6 +1169,16 @@ const EXTENSION_SOURCES = {
   "contentScript-prop": {},
   "contentScript-prop-after-inject": {},
 };
+// When our default content script CSP is applied, only
+// liveSrc: true are loading.  IOW, the "script" test above
+// will fail.
+const EXTENSION_SOURCES_CONTENT_CSP = {
+  contentScript: { liveSrc: true },
+  "contentScript-attr-after-inject": { liveSrc: true },
+  "contentScript-content-inject-after-attr": { liveSrc: true },
+  "contentScript-prop": { liveSrc: true },
+  "contentScript-prop-after-inject": { liveSrc: true },
+};
 // All sources.
 const SOURCES = Object.assign({}, PAGE_SOURCES, EXTENSION_SOURCES);
 
@@ -1161,18 +1201,29 @@ registerStaticPage(
   </html>`
 );
 
+function catchViolation() {
+  // eslint-disable-next-line mozilla/balanced-listeners
+  document.addEventListener("securitypolicyviolation", e => {
+    browser.test.assertTrue(
+      e.documentURI !== "moz-extension",
+      `securitypolicyviolation: ${e.violatedDirective} ${e.documentURI}`
+    );
+  });
+}
+
 const EXTENSION_DATA = {
   manifest: {
     content_scripts: [
       {
         matches: ["http://*/page.html"],
         run_at: "document_start",
-        js: ["content_script.js"],
+        js: ["violation.js", "content_script.js"],
       },
     ],
   },
 
   files: {
+    "violation.js": catchViolation,
     "content_script.js": getInjectionScript(TESTS, {
       source: "contentScript",
       origin: "contentScript",
@@ -1198,7 +1249,7 @@ function mergeSources(a, b) {
 // use in verifying request triggering principals.
 function getOrigins(extension) {
   return {
-    page: Services.scriptSecurityManager.createCodebasePrincipal(pageURI, {})
+    page: Services.scriptSecurityManager.createContentPrincipal(pageURI, {})
       .origin,
     contentScript: Cu.getObjectPrincipal(
       Cu.Sandbox([extension.principal, pageURL])
@@ -1272,3 +1323,51 @@ add_task(async function test_contentscript_csp() {
   await extension.unload();
   await contentPage.close();
 });
+
+/**
+ * Tests that the correct CSP is applied to loads of inline content
+ * depending on whether the load was initiated by an extension or the
+ * content page.
+ */
+add_task(async function test_extension_contentscript_csp() {
+  Services.prefs.setBoolPref("extensions.manifestV3.enabled", true);
+
+  // TODO bug 1408193: We currently don't get the full set of CSP reports when
+  // running in network scheduling chaos mode. It's not entirely clear why.
+  let chaosMode = parseInt(env.get("MOZ_CHAOSMODE"), 16);
+  let checkCSPReports = !(chaosMode === 0 || chaosMode & 0x02);
+
+  gContentSecurityPolicy = `default-src 'none' 'report-sample'; script-src 'nonce-deadbeef' 'unsafe-eval' 'report-sample'; report-uri ${CSP_REPORT_PATH};`;
+
+  let data = {
+    ...EXTENSION_DATA,
+    manifest: {
+      ...EXTENSION_DATA.manifest,
+      manifest_version: 3,
+    },
+  };
+  let extension = ExtensionTestUtils.loadExtension(data);
+  await extension.startup();
+
+  let urlsPromise = extension.awaitMessage("css-sources").then(msg => {
+    return mergeSources(
+      computeExpectedForbiddenURLs(msg, true, true),
+      computeBaseURLs(TESTS, EXTENSION_SOURCES_CONTENT_CSP, PAGE_SOURCES)
+    );
+  });
+
+  let origins = getOrigins(extension.extension);
+
+  let finished = Promise.all([
+    awaitLoads(urlsPromise, origins),
+    checkCSPReports && awaitCSP(urlsPromise),
+  ]);
+
+  let contentPage = await ExtensionTestUtils.loadContentPage(pageURL);
+
+  await finished;
+
+  await extension.unload();
+  await contentPage.close();
+  Services.prefs.clearUserPref("extensions.manifestV3.enabled");
+});
