@@ -4,7 +4,6 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/storage.h"
-#include "mozilla/dom/URLSearchParams.h"
 #include "nsString.h"
 #include "nsUnicharUtils.h"
 #include "nsWhitespaceTokenizer.h"
@@ -18,6 +17,8 @@
 #include "nsPrintfCString.h"
 #include "nsNavHistory.h"
 #include "mozilla/Likely.h"
+#include "mozilla/Utf8.h"
+#include "nsURLHelper.h"
 #include "nsVariant.h"
 
 // Maximum number of chars to search through.
@@ -108,7 +109,7 @@ static MOZ_ALWAYS_INLINE bool isOnBoundary(const_char_iterator aPos) {
 
 /**
  * Check whether a token string matches a particular position of a source
- * string, case insensitively.
+ * string, case insensitively (or optionally, case and diacritic insensitively).
  *
  * @param aTokenStart
  *        An iterator pointing to the start of the token string.
@@ -119,6 +120,8 @@ static MOZ_ALWAYS_INLINE bool isOnBoundary(const_char_iterator aPos) {
  *        matching at.
  * @param aSourceEnd
  *        An iterator pointing past-the-end of the source string.
+ * @param aMatchDiacritics
+ *        Whether or not the match is diacritic-sensitive.
  *
  * @return true if the string [aTokenStart, aTokenEnd) matches the start of
  *         the string [aSourceStart, aSourceEnd, false otherwise.
@@ -126,7 +129,8 @@ static MOZ_ALWAYS_INLINE bool isOnBoundary(const_char_iterator aPos) {
 static MOZ_ALWAYS_INLINE bool stringMatch(const_char_iterator aTokenStart,
                                           const_char_iterator aTokenEnd,
                                           const_char_iterator aSourceStart,
-                                          const_char_iterator aSourceEnd) {
+                                          const_char_iterator aSourceEnd,
+                                          bool aMatchDiacritics) {
   const_char_iterator tokenCur = aTokenStart, sourceCur = aSourceStart;
 
   while (tokenCur < aTokenEnd) {
@@ -136,8 +140,8 @@ static MOZ_ALWAYS_INLINE bool stringMatch(const_char_iterator aTokenStart,
 
     bool error;
     if (!CaseInsensitiveUTF8CharsEqual(sourceCur, tokenCur, aSourceEnd,
-                                       aTokenEnd, &sourceCur, &tokenCur,
-                                       &error)) {
+                                       aTokenEnd, &sourceCur, &tokenCur, &error,
+                                       aMatchDiacritics)) {
       return false;
     }
   }
@@ -173,6 +177,9 @@ static bool findInString(const nsDependentCSubstring& aToken,
     return false;
   }
 
+  const nsNavHistory* history = nsNavHistory::GetConstHistoryService();
+  bool matchDiacritics = history && history->MatchDiacritics();
+
   const_char_iterator tokenStart(aToken.BeginReading()),
       tokenEnd(aToken.EndReading()), tokenNext,
       sourceStart(aSourceString.BeginReading()),
@@ -183,10 +190,15 @@ static bool findInString(const nsDependentCSubstring& aToken,
   if (tokenFirstChar == uint32_t(-1)) {
     return false;
   }
+  if (!matchDiacritics) {
+    tokenFirstChar = ToNaked(tokenFirstChar);
+  }
 
   for (;;) {
-    // Scan forward to the next viable candidate (if any).
-    goToNextSearchCandidate(sourceCur, sourceEnd, tokenFirstChar);
+    if (matchDiacritics) {
+      // Scan forward to the next viable candidate (if any).
+      goToNextSearchCandidate(sourceCur, sourceEnd, tokenFirstChar);
+    }
     if (sourceCur == sourceEnd) {
       break;
     }
@@ -199,11 +211,15 @@ static bool findInString(const nsDependentCSubstring& aToken,
     if (sourceFirstChar == uint32_t(-1)) {
       return false;
     }
+    if (!matchDiacritics) {
+      sourceFirstChar = ToNaked(sourceFirstChar);
+    }
 
     if (sourceFirstChar == tokenFirstChar &&
         (aBehavior != eFindOnBoundary || sourceCur == sourceStart ||
          isOnBoundary(sourceCur)) &&
-        stringMatch(tokenNext, tokenEnd, sourceNext, sourceEnd)) {
+        stringMatch(tokenNext, tokenEnd, sourceNext, sourceEnd,
+                    matchDiacritics)) {
       return true;
     }
 
@@ -218,33 +234,11 @@ getSharedUTF8String(mozIStorageValueArray* aValues, uint32_t aIndex) {
   uint32_t len;
   const char* str = aValues->AsSharedUTF8String(aIndex, &len);
   if (!str) {
-    return nsDependentCString("", (uint32_t)0);
+    return nsDependentCString("", (size_t)0);
   }
   return nsDependentCString(str, len);
 }
 
-class MOZ_STACK_CLASS GetQueryParamIterator final
-    : public URLParams::ForEachIterator {
- public:
-  explicit GetQueryParamIterator(const nsCString& aParamName,
-                                 nsVariant* aResult)
-      : mParamName(aParamName), mResult(aResult) {}
-
-  bool URLParamsIterator(const nsAString& aName,
-                         const nsAString& aValue) override {
-    NS_ConvertUTF16toUTF8 name(aName);
-    if (!mParamName.Equals(name)) {
-      return true;
-    }
-    mResult->SetAsAString(aValue);
-    return false;
-  }
-
- private:
-  const nsCString& mParamName;
-  nsVariant* mResult;
-};
-
 /**
  * Gets the length of the prefix in a URI spec.  "Prefix" is defined to be the
  * scheme, colon, and, if present, two slashes.
@@ -356,8 +350,8 @@ namespace places {
 nsresult MatchAutoCompleteFunction::create(mozIStorageConnection* aDBConn) {
   RefPtr<MatchAutoCompleteFunction> function = new MatchAutoCompleteFunction();
 
-  nsresult rv = aDBConn->CreateFunction(
-      NS_LITERAL_CSTRING("autocomplete_match"), kArgIndexLength, function);
+  nsresult rv = aDBConn->CreateFunction("autocomplete_match"_ns,
+                                        kArgIndexLength, function);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -373,7 +367,7 @@ nsDependentCSubstring MatchAutoCompleteFunction::fixupURISpec(
   // Otherwise, we will simply use our original string.
   bool unescaped = NS_UnescapeURL(aURISpec.BeginReading(), aURISpec.Length(),
                                   esc_SkipControl, aSpecBuf);
-  if (unescaped && IsUTF8(aSpecBuf)) {
+  if (unescaped && IsUtf8(aSpecBuf)) {
     fixedSpec.Rebind(aSpecBuf, 0);
   } else {
     fixedSpec.Rebind(aURISpec, 0);
@@ -382,11 +376,11 @@ nsDependentCSubstring MatchAutoCompleteFunction::fixupURISpec(
   if (aMatchBehavior == mozIPlacesAutoComplete::MATCH_ANYWHERE_UNMODIFIED)
     return fixedSpec;
 
-  if (StringBeginsWith(fixedSpec, NS_LITERAL_CSTRING("http://"))) {
+  if (StringBeginsWith(fixedSpec, "http://"_ns)) {
     fixedSpec.Rebind(fixedSpec, 7);
-  } else if (StringBeginsWith(fixedSpec, NS_LITERAL_CSTRING("https://"))) {
+  } else if (StringBeginsWith(fixedSpec, "https://"_ns)) {
     fixedSpec.Rebind(fixedSpec, 8);
-  } else if (StringBeginsWith(fixedSpec, NS_LITERAL_CSTRING("ftp://"))) {
+  } else if (StringBeginsWith(fixedSpec, "ftp://"_ns)) {
     fixedSpec.Rebind(fixedSpec, 6);
   }
 
@@ -446,9 +440,8 @@ MatchAutoCompleteFunction::OnFunctionCall(mozIStorageValueArray* aArguments,
   // We only want to filter javascript: URLs if we are not supposed to search
   // for them, and the search does not start with "javascript:".
   if (matchBehavior != mozIPlacesAutoComplete::MATCH_ANYWHERE_UNMODIFIED &&
-      StringBeginsWith(url, NS_LITERAL_CSTRING("javascript:")) &&
-      !HAS_BEHAVIOR(JAVASCRIPT) &&
-      !StringBeginsWith(searchString, NS_LITERAL_CSTRING("javascript:"))) {
+      StringBeginsWith(url, "javascript:"_ns) && !HAS_BEHAVIOR(JAVASCRIPT) &&
+      !StringBeginsWith(searchString, "javascript:"_ns)) {
     NS_ADDREF(*_result = mCachedZero);
     return NS_OK;
   }
@@ -535,8 +528,7 @@ MatchAutoCompleteFunction::OnFunctionCall(mozIStorageValueArray* aArguments,
 nsresult CalculateFrecencyFunction::create(mozIStorageConnection* aDBConn) {
   RefPtr<CalculateFrecencyFunction> function = new CalculateFrecencyFunction();
 
-  nsresult rv = aDBConn->CreateFunction(
-      NS_LITERAL_CSTRING("calculate_frecency"), -1, function);
+  nsresult rv = aDBConn->CreateFunction("calculate_frecency"_ns, -1, function);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -592,7 +584,7 @@ CalculateFrecencyFunction::OnFunctionCall(mozIStorageValueArray* aArguments,
     NS_ENSURE_STATE(getPageInfo);
     mozStorageStatementScoper infoScoper(getPageInfo);
 
-    rv = getPageInfo->BindInt64ByName(NS_LITERAL_CSTRING("page_id"), pageId);
+    rv = getPageInfo->BindInt64ByName("page_id"_ns, pageId);
     NS_ENSURE_SUCCESS(rv, rv);
 
     bool hasResult = false;
@@ -620,7 +612,7 @@ CalculateFrecencyFunction::OnFunctionCall(mozIStorageValueArray* aArguments,
         "%d AND %d ", nsINavHistoryService::TRANSITION_REDIRECT_PERMANENT,
         nsINavHistoryService::TRANSITION_REDIRECT_TEMPORARY);
     nsCOMPtr<mozIStorageStatement> getVisits = DB->GetStatement(
-        NS_LITERAL_CSTRING(
+        nsLiteralCString(
             "/* do not warn (bug 659740 - SQLite may ignore index if few "
             "visits exist) */"
             "SELECT "
@@ -632,18 +624,18 @@ CalculateFrecencyFunction::OnFunctionCall(mozIStorageValueArray* aArguments,
             "LEFT JOIN moz_historyvisits origin ON origin.id = v.from_visit "
             "AND v.visit_type BETWEEN ") +
         redirectsTransitionFragment +
-        NS_LITERAL_CSTRING(
+        nsLiteralCString(
             "LEFT JOIN moz_historyvisits target ON v.id = target.from_visit "
             "AND target.visit_type BETWEEN ") +
         redirectsTransitionFragment +
-        NS_LITERAL_CSTRING("WHERE v.place_id = :page_id "
-                           "ORDER BY v.visit_date DESC "
-                           "LIMIT :max_visits "));
+        nsLiteralCString("WHERE v.place_id = :page_id "
+                         "ORDER BY v.visit_date DESC "
+                         "LIMIT :max_visits "));
     NS_ENSURE_STATE(getVisits);
     mozStorageStatementScoper visitsScoper(getVisits);
-    rv = getVisits->BindInt64ByName(NS_LITERAL_CSTRING("page_id"), pageId);
+    rv = getVisits->BindInt64ByName("page_id"_ns, pageId);
     NS_ENSURE_SUCCESS(rv, rv);
-    rv = getVisits->BindInt32ByName(NS_LITERAL_CSTRING("max_visits"),
+    rv = getVisits->BindInt32ByName("max_visits"_ns,
                                     history->GetNumVisitsForFrecency());
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -745,8 +737,7 @@ CalculateFrecencyFunction::OnFunctionCall(mozIStorageValueArray* aArguments,
 /* static */
 nsresult GenerateGUIDFunction::create(mozIStorageConnection* aDBConn) {
   RefPtr<GenerateGUIDFunction> function = new GenerateGUIDFunction();
-  nsresult rv =
-      aDBConn->CreateFunction(NS_LITERAL_CSTRING("generate_guid"), 0, function);
+  nsresult rv = aDBConn->CreateFunction("generate_guid"_ns, 0, function);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -771,8 +762,7 @@ GenerateGUIDFunction::OnFunctionCall(mozIStorageValueArray* aArguments,
 /* static */
 nsresult IsValidGUIDFunction::create(mozIStorageConnection* aDBConn) {
   RefPtr<IsValidGUIDFunction> function = new IsValidGUIDFunction();
-  return aDBConn->CreateFunction(NS_LITERAL_CSTRING("is_valid_guid"), 1,
-                                 function);
+  return aDBConn->CreateFunction("is_valid_guid"_ns, 1, function);
 }
 
 NS_IMPL_ISUPPORTS(IsValidGUIDFunction, mozIStorageFunction)
@@ -798,8 +788,7 @@ IsValidGUIDFunction::OnFunctionCall(mozIStorageValueArray* aArguments,
 /* static */
 nsresult GetUnreversedHostFunction::create(mozIStorageConnection* aDBConn) {
   RefPtr<GetUnreversedHostFunction> function = new GetUnreversedHostFunction();
-  nsresult rv = aDBConn->CreateFunction(
-      NS_LITERAL_CSTRING("get_unreversed_host"), 1, function);
+  nsresult rv = aDBConn->CreateFunction("get_unreversed_host"_ns, 1, function);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -824,7 +813,7 @@ GetUnreversedHostFunction::OnFunctionCall(mozIStorageValueArray* aArguments,
     ReverseString(src, dest);
     result->SetAsAString(dest);
   } else {
-    result->SetAsAString(EmptyString());
+    result->SetAsAString(u""_ns);
   }
   result.forget(_result);
   return NS_OK;
@@ -836,8 +825,7 @@ GetUnreversedHostFunction::OnFunctionCall(mozIStorageValueArray* aArguments,
 /* static */
 nsresult FixupURLFunction::create(mozIStorageConnection* aDBConn) {
   RefPtr<FixupURLFunction> function = new FixupURLFunction();
-  nsresult rv =
-      aDBConn->CreateFunction(NS_LITERAL_CSTRING("fixup_url"), 1, function);
+  nsresult rv = aDBConn->CreateFunction("fixup_url"_ns, 1, function);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -856,15 +844,15 @@ FixupURLFunction::OnFunctionCall(mozIStorageValueArray* aArguments,
 
   RefPtr<nsVariant> result = new nsVariant();
 
-  if (StringBeginsWith(src, NS_LITERAL_STRING("http://")))
+  if (StringBeginsWith(src, u"http://"_ns))
     src.Cut(0, 7);
-  else if (StringBeginsWith(src, NS_LITERAL_STRING("https://")))
+  else if (StringBeginsWith(src, u"https://"_ns))
     src.Cut(0, 8);
-  else if (StringBeginsWith(src, NS_LITERAL_STRING("ftp://")))
+  else if (StringBeginsWith(src, u"ftp://"_ns))
     src.Cut(0, 6);
 
   // Remove common URL hostname prefixes
-  if (StringBeginsWith(src, NS_LITERAL_STRING("www."))) {
+  if (StringBeginsWith(src, u"www."_ns)) {
     src.Cut(0, 4);
   }
 
@@ -873,55 +861,6 @@ FixupURLFunction::OnFunctionCall(mozIStorageValueArray* aArguments,
   return NS_OK;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-//// Frecency Changed Notification Function
-
-/* static */
-nsresult FrecencyNotificationFunction::create(mozIStorageConnection* aDBConn) {
-  RefPtr<FrecencyNotificationFunction> function =
-      new FrecencyNotificationFunction();
-  nsresult rv = aDBConn->CreateFunction(NS_LITERAL_CSTRING("notify_frecency"),
-                                        5, function);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  return NS_OK;
-}
-
-NS_IMPL_ISUPPORTS(FrecencyNotificationFunction, mozIStorageFunction)
-
-NS_IMETHODIMP
-FrecencyNotificationFunction::OnFunctionCall(mozIStorageValueArray* aArgs,
-                                             nsIVariant** _result) {
-  uint32_t numArgs;
-  nsresult rv = aArgs->GetNumEntries(&numArgs);
-  NS_ENSURE_SUCCESS(rv, rv);
-  MOZ_ASSERT(numArgs == 5);
-
-  int32_t newFrecency = aArgs->AsInt32(0);
-
-  nsAutoCString spec;
-  rv = aArgs->GetUTF8String(1, spec);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsAutoCString guid;
-  rv = aArgs->GetUTF8String(2, guid);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  bool hidden = static_cast<bool>(aArgs->AsInt32(3));
-  PRTime lastVisitDate = static_cast<PRTime>(aArgs->AsInt64(4));
-
-  const nsNavHistory* navHistory = nsNavHistory::GetConstHistoryService();
-  NS_ENSURE_STATE(navHistory);
-  navHistory->DispatchFrecencyChangedNotification(spec, newFrecency, guid,
-                                                  hidden, lastVisitDate);
-
-  RefPtr<nsVariant> result = new nsVariant();
-  rv = result->SetAsInt32(newFrecency);
-  NS_ENSURE_SUCCESS(rv, rv);
-  result.forget(_result);
-  return NS_OK;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 //// Store Last Inserted Id Function
 
@@ -929,8 +868,8 @@ FrecencyNotificationFunction::OnFunctionCall(mozIStorageValueArray* aArgs,
 nsresult StoreLastInsertedIdFunction::create(mozIStorageConnection* aDBConn) {
   RefPtr<StoreLastInsertedIdFunction> function =
       new StoreLastInsertedIdFunction();
-  nsresult rv = aDBConn->CreateFunction(
-      NS_LITERAL_CSTRING("store_last_inserted_id"), 2, function);
+  nsresult rv =
+      aDBConn->CreateFunction("store_last_inserted_id"_ns, 2, function);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -978,8 +917,7 @@ StoreLastInsertedIdFunction::OnFunctionCall(mozIStorageValueArray* aArgs,
 /* static */
 nsresult GetQueryParamFunction::create(mozIStorageConnection* aDBConn) {
   RefPtr<GetQueryParamFunction> function = new GetQueryParamFunction();
-  return aDBConn->CreateFunction(NS_LITERAL_CSTRING("get_query_param"), 2,
-                                 function);
+  return aDBConn->CreateFunction("get_query_param"_ns, 2, function);
 }
 
 NS_IMPL_ISUPPORTS(GetQueryParamFunction, mozIStorageFunction)
@@ -995,8 +933,16 @@ GetQueryParamFunction::OnFunctionCall(mozIStorageValueArray* aArguments,
 
   RefPtr<nsVariant> result = new nsVariant();
   if (!queryString.IsEmpty() && !paramName.IsEmpty()) {
-    GetQueryParamIterator iterator(paramName, result);
-    URLParams::Parse(queryString, iterator);
+    URLParams::Parse(
+        queryString,
+        [&paramName, &result](const nsAString& aName, const nsAString& aValue) {
+          NS_ConvertUTF16toUTF8 name(aName);
+          if (!paramName.Equals(name)) {
+            return true;
+          }
+          result->SetAsAString(aValue);
+          return false;
+        });
   }
 
   result.forget(_result);
@@ -1009,7 +955,7 @@ GetQueryParamFunction::OnFunctionCall(mozIStorageValueArray* aArguments,
 /* static */
 nsresult HashFunction::create(mozIStorageConnection* aDBConn) {
   RefPtr<HashFunction> function = new HashFunction();
-  return aDBConn->CreateFunction(NS_LITERAL_CSTRING("hash"), -1, function);
+  return aDBConn->CreateFunction("hash"_ns, -1, function);
 }
 
 NS_IMPL_ISUPPORTS(HashFunction, mozIStorageFunction)
@@ -1049,8 +995,7 @@ HashFunction::OnFunctionCall(mozIStorageValueArray* aArguments,
 /* static */
 nsresult GetPrefixFunction::create(mozIStorageConnection* aDBConn) {
   RefPtr<GetPrefixFunction> function = new GetPrefixFunction();
-  nsresult rv =
-      aDBConn->CreateFunction(NS_LITERAL_CSTRING("get_prefix"), 1, function);
+  nsresult rv = aDBConn->CreateFunction("get_prefix"_ns, 1, function);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -1082,8 +1027,7 @@ GetPrefixFunction::OnFunctionCall(mozIStorageValueArray* aArgs,
 /* static */
 nsresult GetHostAndPortFunction::create(mozIStorageConnection* aDBConn) {
   RefPtr<GetHostAndPortFunction> function = new GetHostAndPortFunction();
-  nsresult rv = aDBConn->CreateFunction(NS_LITERAL_CSTRING("get_host_and_port"),
-                                        1, function);
+  nsresult rv = aDBConn->CreateFunction("get_host_and_port"_ns, 1, function);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -1120,8 +1064,8 @@ nsresult StripPrefixAndUserinfoFunction::create(
     mozIStorageConnection* aDBConn) {
   RefPtr<StripPrefixAndUserinfoFunction> function =
       new StripPrefixAndUserinfoFunction();
-  nsresult rv = aDBConn->CreateFunction(
-      NS_LITERAL_CSTRING("strip_prefix_and_userinfo"), 1, function);
+  nsresult rv =
+      aDBConn->CreateFunction("strip_prefix_and_userinfo"_ns, 1, function);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -1156,8 +1100,7 @@ StripPrefixAndUserinfoFunction::OnFunctionCall(mozIStorageValueArray* aArgs,
 nsresult IsFrecencyDecayingFunction::create(mozIStorageConnection* aDBConn) {
   RefPtr<IsFrecencyDecayingFunction> function =
       new IsFrecencyDecayingFunction();
-  nsresult rv = aDBConn->CreateFunction(
-      NS_LITERAL_CSTRING("is_frecency_decaying"), 0, function);
+  nsresult rv = aDBConn->CreateFunction("is_frecency_decaying"_ns, 0, function);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -1185,48 +1128,13 @@ IsFrecencyDecayingFunction::OnFunctionCall(mozIStorageValueArray* aArgs,
   return NS_OK;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-//// sqrt function
-
-/* static */
-nsresult SqrtFunction::create(mozIStorageConnection* aDBConn) {
-  RefPtr<SqrtFunction> function = new SqrtFunction();
-  nsresult rv =
-      aDBConn->CreateFunction(NS_LITERAL_CSTRING("sqrt"), 1, function);
-  NS_ENSURE_SUCCESS(rv, rv);
-  return NS_OK;
-}
-
-NS_IMPL_ISUPPORTS(SqrtFunction, mozIStorageFunction)
-
-NS_IMETHODIMP
-SqrtFunction::OnFunctionCall(mozIStorageValueArray* aArgs,
-                             nsIVariant** _result) {
-  MOZ_ASSERT(aArgs);
-
-  uint32_t numArgs;
-  nsresult rv = aArgs->GetNumEntries(&numArgs);
-  NS_ENSURE_SUCCESS(rv, rv);
-  MOZ_ASSERT(numArgs == 1);
-
-  double value = aArgs->AsDouble(0);
-
-  RefPtr<nsVariant> result = new nsVariant();
-  rv = result->SetAsDouble(sqrt(value));
-  NS_ENSURE_SUCCESS(rv, rv);
-  result.forget(_result);
-
-  return NS_OK;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 //// Note Sync Change Function
 
 /* static */
 nsresult NoteSyncChangeFunction::create(mozIStorageConnection* aDBConn) {
   RefPtr<NoteSyncChangeFunction> function = new NoteSyncChangeFunction();
-  nsresult rv = aDBConn->CreateFunction(NS_LITERAL_CSTRING("note_sync_change"),
-                                        0, function);
+  nsresult rv = aDBConn->CreateFunction("note_sync_change"_ns, 0, function);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -1242,5 +1150,29 @@ NoteSyncChangeFunction::OnFunctionCall(mozIStorageValueArray* aArgs,
   return NS_OK;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+//// Invalidate days of history Function
+
+/* static */
+nsresult InvalidateDaysOfHistoryFunction::create(
+    mozIStorageConnection* aDBConn) {
+  RefPtr<InvalidateDaysOfHistoryFunction> function =
+      new InvalidateDaysOfHistoryFunction();
+  nsresult rv =
+      aDBConn->CreateFunction("invalidate_days_of_history"_ns, 0, function);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return NS_OK;
+}
+
+NS_IMPL_ISUPPORTS(InvalidateDaysOfHistoryFunction, mozIStorageFunction)
+
+NS_IMETHODIMP
+InvalidateDaysOfHistoryFunction::OnFunctionCall(mozIStorageValueArray* aArgs,
+                                                nsIVariant** _result) {
+  nsNavHistory::InvalidateDaysOfHistory();
+  return NS_OK;
+}
+
 }  // namespace places
 }  // namespace mozilla
