@@ -13,6 +13,12 @@
 
 namespace PingSender {
 
+// System-specific function that changes the current working directory to be
+// the same as the one containing the ping file. This is currently required on
+// Windows to release the Firefox installation folder (see bug 1597803 for more
+// details) and is a no-op on other platforms.
+void ChangeCurrentWorkingDirectory(const std::string& pingPath);
+
 // System-specific function to make an HTTP POST operation
 bool Post(const std::string& url, const std::string& payload);
 
